/**
 * Copyright (c) 2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.bcgateway;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.cli.api.command.AbstractCommand;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;

public abstract class BcGatewayAbstractCommand extends AbstractCommand {

    private static final String COMMAND_PREFIX = "bc-gateway.";

    public static final String DEFAULT_COMPONENT_ID = "petals-bc-gateway";

    /**
     * The option associated to the component identifier
     */
    public static final String COMPONENT_ID_SHORT_OPTION = "n";

    private static final String COMPONENT_ID_LONG_OPTION = "component";

    private static final String COMPONENT_ID_ARG_NAME = "component-identifier";

    private static final Option COMPONENT_ID_OPTION = Option.builder(COMPONENT_ID_SHORT_OPTION)
            .longOpt(COMPONENT_ID_LONG_OPTION).required(false).numberOfArgs(1).argName(COMPONENT_ID_ARG_NAME)
            .desc("The component identifier associated to the BC Gateway.").build();

    public BcGatewayAbstractCommand(final String name, final String description) {
        super(COMMAND_PREFIX + name);
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription(description);
        this.setOptionsDescription(CommandUtil.formatCommandOptionsDescription(this));
    }

    @Override
    public Options createOptions() {
        return super.createOptions().addOption(COMPONENT_ID_OPTION);
    }

    @Override
    public boolean isConnectionRequired() {
        return true;
    }

    @Override
    public final void doExecute(final String[] args) throws CommandException {
        final CommandLineParser parser = new DefaultParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);

            // parse throws an UnrecognizedOptionException if there is an options starting with "-" that is unknown, but
            // arguments that are not understood are simply added to the args of CLI. In our case, we don't want any or
            // there is an error somewhere.
            if (cli.getArgs().length > 0) {
                throw new CommandBadArgumentNumberException(this);
            }

            final String componentId;
            if (cli.hasOption(COMPONENT_ID_SHORT_OPTION)) {
                componentId = cli.getOptionValue(COMPONENT_ID_SHORT_OPTION);
            } else {
                componentId = DEFAULT_COMPONENT_ID;
            }

            doExecute(cli, componentId);

        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        }
    }

    protected abstract void doExecute(CommandLine cli, String componentId) throws CommandException;
}
