/**
 * Copyright (c) 2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.bcgateway;

import java.io.PrintStream;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.ow2.petals.admin.jmx.JMXClientConnection;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.binding.gateway.clientserver.api.AdminRuntimeService;
import org.ow2.petals.binding.gateway.clientserver.api.AdminService;
import org.ow2.petals.binding.gateway.clientserver.api.CommonAdminService;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidArgumentException;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.configuration.component.exception.ConfigurationComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.InstallerComponentDoesNotExistException;

/**
 * The command 'bc-gateway.list-transport-listener'
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ListTransportListener extends BcGatewayAbstractCommand {

    /**
     * The command name
     */
    private static final String NAME = "list-transport-listener";

    /**
     * The command description
     */
    private static final String DESCRIPTION = "List transport listeners of the given BC Gateway";

    /**
     * The option associated to the transport listener identifier
     */
    public static final String IDENTIFIER_SHORT_OPTION = "i";

    private static final String IDENTIFIER_LONG_OPTION = "identifier";

    private static final String IDENTIFIER_ARG_NAME = "transport-listener-identifier";

    private static final Option IDENTIFIER_OPTION = Option.builder(IDENTIFIER_SHORT_OPTION)
            .longOpt(IDENTIFIER_LONG_OPTION).required(false).numberOfArgs(1).argName(IDENTIFIER_ARG_NAME)
            .desc("The unique identifier of the transport listener to list.").build();

    /**
     * The option associated to the transport listener port
     */
    public static final String PORT_SHORT_OPTION = "p";

    private static final String PORT_LONG_OPTION = "port";

    private static final String PORT_ARG_NAME = "listening-port";

    private static final Option PORT_OPTION = Option.builder(PORT_SHORT_OPTION).longOpt(PORT_LONG_OPTION)
            .required(false).numberOfArgs(1).argName(PORT_ARG_NAME)
            .desc("The listening port of the transport listeners to list.").build();

    /**
     * The option associated to the transport status
     */
    public static final String LISTENING_SHORT_OPTION = "l";

    private static final String LISTENING_LONG_OPTION = "listening";

    private static final String LISTENING_ARG_NAME = "boolean";

    private static final Option LISTENING_OPTION = Option.builder(LISTENING_SHORT_OPTION).longOpt(LISTENING_LONG_OPTION)
            .required(false).numberOfArgs(1).optionalArg(true).argName(LISTENING_ARG_NAME)
            .desc("The listening status of the transport listeners to list.").build();

    public ListTransportListener() {
        super(NAME, DESCRIPTION);
    }

    @Override
    public Options createOptions() {
        return super.createOptions().addOption(IDENTIFIER_OPTION).addOption(PORT_OPTION).addOption(LISTENING_OPTION);
    }

    @Override
    protected void doExecute(final CommandLine cli, final String componentId) throws CommandException {
        try {
            final String transportListenerId;
            if (cli.hasOption(IDENTIFIER_SHORT_OPTION)) {
                transportListenerId = cli.getOptionValue(IDENTIFIER_SHORT_OPTION);
            } else {
                transportListenerId = null;
            }

            final Integer port;
            if (cli.hasOption(PORT_SHORT_OPTION)) {
                final String portStr = cli.getOptionValue(PORT_SHORT_OPTION);
                try {
                    port = Integer.valueOf(portStr);
                } catch (final NumberFormatException e) {
                    throw new CommandInvalidArgumentException(this, PORT_OPTION, portStr, e);
                }
            } else {
                port = null;
            }
            
            final Boolean listening;
            if (cli.hasOption(LISTENING_SHORT_OPTION)) {
                final String listeningStr = cli.getOptionValue(LISTENING_LONG_OPTION);
                // null means there is no arg aka true by default
                if (listeningStr == null || listeningStr.equalsIgnoreCase("true")) {
                    listening = true;
                } else if (listeningStr.equalsIgnoreCase("false")) {
                    listening = false;
                } else {
                    throw new CommandInvalidArgumentException(this, LISTENING_OPTION, listeningStr);
                }
            } else {
                listening = null;
            }

            CommonAdminService adminServiceClient;
            final JMXClient client = JMXClientConnection.getJMXClient();
            try {
                // First, try using the installation configuration component
                adminServiceClient = client.getInstallerComponentClient(componentId)
                        .getInstallationConfigurationClient(AdminService.class);
            } catch (final NullPointerException | InstallerComponentDoesNotExistException
                    | ConfigurationComponentDoesNotExistException e) {
                // 2nd, try using the runtime configuration component
                adminServiceClient = client.getComponentClient(componentId)
                        .getRuntimeConfigurationClient(AdminRuntimeService.class);
            }

            final Map<String, Object[]> transportListeners = adminServiceClient.getTransportListeners();

            final Iterator<Entry<String, Object[]>> itTransportListeners = transportListeners.entrySet().iterator();
            while (itTransportListeners.hasNext()) {
                final Entry<String, Object[]> entry = itTransportListeners.next();
                final Object[] values = entry.getValue();
                if (transportListenerId != null && !transportListenerId.equals(entry.getKey())) {
                    itTransportListeners.remove();
                } else if (port != null && !port.equals(values[0])) {
                    itTransportListeners.remove();
                } else if (listening != null && listening != values[1]) {
                    itTransportListeners.remove();
                }
            }

            final StringBuilder transportListenerContent = new StringBuilder();
            for (final Entry<String, Object[]> transportListener : transportListeners.entrySet()) {
                final Object[] values = transportListener.getValue();
                final Integer p = (Integer) values[0];
                final Boolean l = (Boolean) values[1];
                final String bindingError = (String) values[2];
                transportListenerContent
                        .append(String.format("%s on port %d: %s%s%n", transportListener.getKey(), p,
                                l ? "listening" : "not listening",
                                bindingError.isEmpty() ? "" : " (" + bindingError + ")"));
            }
            final PrintStream out = this.getShell().getPrintStream();
            out.print(transportListenerContent);

        } catch (final PetalsException e) {
            throw new CommandException(this, e);
        }
    }
}
