/**
 * Copyright (c) 2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.bcgateway;

import java.net.UnknownHostException;

import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.admin.jmx.JMXClientConnection;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.connection.exception.HostMissingException;
import org.ow2.petals.cli.api.connection.exception.InvalidPortException;
import org.ow2.petals.cli.api.connection.exception.PasswordMissingException;
import org.ow2.petals.cli.api.connection.exception.UsernameMissingException;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.cli.extension.bcgateway.mock.PetalsBcGatewayInstallationMock;
import org.ow2.petals.cli.extension.bcgateway.mock.PetalsBcGatewayRuntimeConfigurationMock;
import org.ow2.petals.cli.shell.command.Connect;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.exception.DuplicatedServiceException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.MissingServiceException;
import org.ow2.petals.jmx.api.mock.junit.PetalsJmxApiJunitRule.ComponentType;

/**
 * Unit tests of the command 'bc-gateway.add-transport-listener'
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class AddTransportListenerTest extends BcGatewayAbstractCommandTest {

    @Before
    public void setUp() {
        this.command = new AddTransportListener();
    }

    @Override
    public void componentIdentifierOptionMissing() throws Exception {
        // disable because it makes no sense here: this commands has required options so the wrong exception will be
        // thrown
    }

    /**
     * <p>
     * Test of the command "bc-gateway.add-transport-listener" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about required arguments missing.
     * </p>
     */
    @Test
    public void noArgumentProvided() throws ConnectionErrorException, DuplicatedServiceException,
            MissingServiceException, CommandException, InstallationServiceErrorException, UnknownHostException,
            HostMissingException, InvalidPortException, UsernameMissingException, PasswordMissingException {

        // Set the environment
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();

        // Execute command
        command.setShell(shell);
        try {
            command.execute(new String[] {});
            fail("CommandMissingOptionsException is not thrown");
        } catch (final CommandMissingOptionsException e) {
            assertCommandMissingOptionsException(e, AddTransportListener.IDENTIFIER_SHORT_OPTION,
                    AddTransportListener.PORT_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "bc-gateway.add-transport-listener" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>only the option about transport listener identifier is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about required arguments missing.
     * </p>
     */
    @Test
    public void onlyIdentifierProvided() throws ConnectionErrorException, DuplicatedServiceException,
            MissingServiceException, CommandException, InstallationServiceErrorException, UnknownHostException,
            HostMissingException, InvalidPortException, UsernameMissingException, PasswordMissingException {

        // Set the environment
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();

        // Execute command
        command.setShell(shell);
        try {
            command.execute(new String[] { "-" + AddTransportListener.IDENTIFIER_SHORT_OPTION, "an-identifier" });
            fail("CommandMissingOptionsException is not thrown");
        } catch (final CommandMissingOptionsException e) {
            assertCommandMissingOptionsException(e, AddTransportListener.PORT_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "bc-gateway.add-transport-listener" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>only the option about transport listener port is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about required arguments missing.
     * </p>
     */
    @Test
    public void onlyPortProvided() throws ConnectionErrorException, DuplicatedServiceException, MissingServiceException,
            CommandException, InstallationServiceErrorException, UnknownHostException, HostMissingException,
            InvalidPortException, UsernameMissingException, PasswordMissingException {

        // Set the environment
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();

        // Execute command
        command.setShell(shell);
        try {
            command.execute(new String[] { "-" + AddTransportListener.PORT_SHORT_OPTION, "40000" });
            fail("CommandMissingOptionsException is not thrown");
        } catch (final CommandMissingOptionsException e) {
            assertCommandMissingOptionsException(e, AddTransportListener.IDENTIFIER_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "bc-gateway.add-transport-listener" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>the option about transport listener identifier is provided</li>
     * <li>the option about transport listener port is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the invalid port value.
     * </p>
     */
    @Test
    public void invalidPortProvided() throws ConnectionErrorException, DuplicatedServiceException,
            MissingServiceException, CommandException, InstallationServiceErrorException, UnknownHostException,
            HostMissingException, InvalidPortException, UsernameMissingException, PasswordMissingException {

        // Set the environment
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();

        // Execute command
        final String invalidValue = "invalid-value";
        command.setShell(shell);
        try {
            command.execute(new String[] { "-" + AddTransportListener.IDENTIFIER_SHORT_OPTION, "an-identifier",
                    "-" + AddTransportListener.PORT_SHORT_OPTION, "invalid-value" });
            fail("CommandInvalidArgumentException is not thrown");
        } catch (final CommandInvalidArgumentException e) {
            assertCommandInvalidArgumentException(e, AddTransportListener.PORT_SHORT_OPTION, invalidValue);
        }
    }

    /**
     * <p>
     * Test of the command "bc-gateway.add-transport-listener" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>a valid transport listener identifier is provided,</li>
     * <li>a valid transport listener port is provided,</li>
     * <li>a valid component identifier is provided,</li>
     * <li>the BC Gateway is in state "STARTED"</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs, the transport listener is correctly added on the right component.
     * </p>
     */
    @Test
    public void componentIdentifierProvided_onStartedGateway()
            throws ConnectionErrorException, DuplicatedServiceException, MissingServiceException, CommandException,
            InstallationServiceErrorException, UnknownHostException, HostMissingException, InvalidPortException,
            UsernameMissingException, PasswordMissingException {

        // Set the environment
        final String anotherComponentIdentifier = "component-B";
        final PetalsBcGatewayRuntimeConfigurationMock petalsBcGatewayRuntimeConfigurationMockDefault = new PetalsBcGatewayRuntimeConfigurationMock();
        final PetalsBcGatewayRuntimeConfigurationMock petalsBcGatewayRuntimeConfigurationMockOther = new PetalsBcGatewayRuntimeConfigurationMock();
        this.jmxApi.addComponentClient(BcGatewayAbstractCommand.DEFAULT_COMPONENT_ID,
                ComponentType.BINDING, null, null, petalsBcGatewayRuntimeConfigurationMockDefault, null);
        this.jmxApi.addComponentClient(anotherComponentIdentifier, ComponentType.BINDING, null,
                null, petalsBcGatewayRuntimeConfigurationMockOther, null);
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();
        assertEquals("Transport listener already configured on the default component", 0,
                petalsBcGatewayRuntimeConfigurationMockDefault.getTransportListeners().size());
        assertEquals("Transport listener already configured on the other component", 0,
                petalsBcGatewayRuntimeConfigurationMockOther.getTransportListeners().size());

        // Execute command
        final String transportListenerId = "tl-B";
        final Integer transportListenerPort = Integer.valueOf(45623);
        command.setShell(shell);
        command.execute(new String[] { "-" + AddTransportListener.IDENTIFIER_SHORT_OPTION, transportListenerId,
                "-" + AddTransportListener.PORT_SHORT_OPTION, transportListenerPort.toString(),
                "-" + AddTransportListener.COMPONENT_ID_SHORT_OPTION, anotherComponentIdentifier });

        // Assertions
        assertTrue("An error message is printed", shell.getError().isEmpty());
        assertEquals("Transport listener added on the default component", 0,
                petalsBcGatewayRuntimeConfigurationMockDefault.getTransportListeners().size());
        assertEquals("Transport listener not added on the right component", 1,
                petalsBcGatewayRuntimeConfigurationMockOther.getTransportListeners().size());
        assertEquals("Unexpected transport listener identifier/port", transportListenerPort,
                petalsBcGatewayRuntimeConfigurationMockOther.getTransportListeners()
                        .get(transportListenerId)[0]);
    }

    /**
     * <p>
     * Test of the command "bc-gateway.add-transport-listener" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>a valid transport listener identifier is provided,</li>
     * <li>a valid transport listener port is provided,</li>
     * <li>a valid component identifier is provided,</li>
     * <li>the BC Gateway is in state "LOADED"</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs, the transport listener is correctly added on the right component.
     * </p>
     */
    @Test
    public void componentIdentifierProvided_onLoadedGateway()
            throws ConnectionErrorException, DuplicatedServiceException, MissingServiceException, CommandException,
            InstallationServiceErrorException, UnknownHostException, HostMissingException, InvalidPortException,
            UsernameMissingException, PasswordMissingException {

        // Set the environment
        final String anotherComponentIdentifier = "component-B";
        final PetalsBcGatewayInstallationMock petalsBcGatewayInstallationConfigurationMockDefault = new PetalsBcGatewayInstallationMock();
        final PetalsBcGatewayInstallationMock petalsBcGatewayInstallationConfigurationMockOther = new PetalsBcGatewayInstallationMock();
        this.jmxApi.addComponentInstallerClient(BcGatewayAbstractCommand.DEFAULT_COMPONENT_ID,
                ComponentType.BINDING, null, petalsBcGatewayInstallationConfigurationMockDefault);
        this.jmxApi.addComponentInstallerClient(anotherComponentIdentifier, ComponentType.BINDING,
                null, petalsBcGatewayInstallationConfigurationMockOther);
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();
        assertEquals("Transport listener already configured on the default component", 0,
                petalsBcGatewayInstallationConfigurationMockDefault.getTransportListeners().size());
        assertEquals("Transport listener already configured on the other component", 0,
                petalsBcGatewayInstallationConfigurationMockOther.getTransportListeners().size());

        // Execute command
        final String transportListenerId = "tl-B";
        final Integer transportListenerPort = Integer.valueOf(45623);
        command.setShell(shell);
        command.execute(new String[] { "-" + AddTransportListener.IDENTIFIER_SHORT_OPTION, transportListenerId,
                "-" + AddTransportListener.PORT_SHORT_OPTION, transportListenerPort.toString(),
                "-" + AddTransportListener.COMPONENT_ID_SHORT_OPTION, anotherComponentIdentifier });

        // Assertions
        assertTrue("An error message is printed", shell.getError().isEmpty());
        assertEquals("Transport listener added on the default component", 0,
                petalsBcGatewayInstallationConfigurationMockDefault.getTransportListeners().size());
        assertEquals("Transport listener not added on the right component", 1,
                petalsBcGatewayInstallationConfigurationMockOther.getTransportListeners().size());
        assertEquals("Unexpected transport listener identifier/port", transportListenerPort,
                petalsBcGatewayInstallationConfigurationMockOther.getTransportListeners()
                        .get(transportListenerId)[0]);
    }

    /**
     * <p>
     * Test of the command "bc-gateway.add-transport-listener" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>the option '-i' is given but without value</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing value.
     * </p>
     */
    @Test
    public void transportListenerIdentifierValueMissing() throws ConnectionErrorException, DuplicatedServiceException,
            MissingServiceException, CommandException, InstallationServiceErrorException, UnknownHostException,
            HostMissingException, InvalidPortException, UsernameMissingException, PasswordMissingException {

        // Set the environment
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();

        // Execute command
        command.setShell(shell);
        try {
            command.execute(new String[] { "-" + AddTransportListener.IDENTIFIER_SHORT_OPTION });
        } catch (final CommandMissingArgumentException e) {
            assertCommandMissingArgumentException(e, AddTransportListener.IDENTIFIER_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "bc-gateway.add-transport-listener" against argument management, where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>the option '-p' is given but without value</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing value.
     * </p>
     */
    @Test
    public void transportListenerPortValueMissing() throws ConnectionErrorException, DuplicatedServiceException,
            MissingServiceException, CommandException, InstallationServiceErrorException, UnknownHostException,
            HostMissingException, InvalidPortException, UsernameMissingException, PasswordMissingException {

        // Set the environment
        this.adminApi.registerNullDomain();
        final StringStreamShell shell = new StringStreamShell();
        final Connect connect = new Connect();
        connect.setShell(shell);
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);
        JMXClientConnection.jmxClient = this.jmxApi.connect();

        // Execute command
        final Command command = new AddTransportListener();
        command.setShell(shell);
        try {
            command.execute(new String[] { "-" + AddTransportListener.PORT_SHORT_OPTION });
        } catch (final CommandMissingArgumentException e) {
            assertCommandMissingArgumentException(e, AddTransportListener.PORT_SHORT_OPTION);
        }
    }
}
